package com.kccoder.fracture;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

class ThreadedForEach<E> {
	private Iterator<E> iterator;
	private IProcessor<E> processor;
	
	public ThreadedForEach(E[] elements, IProcessor<E> processor) {
		this(Arrays.asList(elements), processor);
	}
	
	public ThreadedForEach(Iterable<E> iterable, IProcessor<E> processor) {
		this(iterable.iterator(), processor);
	}
	
	public ThreadedForEach(Iterator<E> iterator, IProcessor<E> processor) {
		this.iterator = iterator;
		this.processor = processor;
	}
	
	public void loop() {
		int coreCount = getCoreCount();
//		System.out.println("Starting " + coreCount + " cores");
		Set<Thread> set = new HashSet<Thread>();
		for(int i = 0; i < coreCount; i++) {
			set.add(startThread());
		}
		
		while(true) {
			Iterator<Thread> iterator = set.iterator();
			while(iterator.hasNext()) {
				Thread t = iterator.next();
				if (!t.isAlive()) {
					iterator.remove();
				}
			}
			if (set.isEmpty()) {return;}
			try {Thread.sleep(5);}catch(Exception e) {}
		}		
	}
	
	private int getCoreCount() {
		int count = getConfiguredCoreCount();
		int pc = getProcessorCount();
		if (count > pc) {
			return pc;
		} else {
			return (count < 1)?1:count;
		}
	}
	
	private int getConfiguredCoreCount() {
		CoreMode cr = Fracture.getCoreMode();
		
		if (cr == CoreMode.ALL_CORES) {
			return getProcessorCount();
		} else if (cr == CoreMode.ALL_CORES_BUT_ONE) {
			return getProcessorCount()-1;
		} else if (cr == CoreMode.ONE_CORE) {
			return 1;
		} else if (cr == CoreMode.TWO_CORES) {
			return 2;
		} else if (cr == CoreMode.THREE_CORES) {
			return 3;
		} else if (cr == CoreMode.FOUR_CORES) {
			return 4;
		} else {
			return 1;
		}
	}
	
	private int getProcessorCount() {
		return Runtime.getRuntime().availableProcessors();
	}
	
	private Thread startThread() {
		Thread t = new Thread(new MyRunnable());
		t.setDaemon(true);
		t.start();
		return t;
	}
	
	private class MyRunnable implements Runnable {
		public void run() {
			while(true) {
				E e = null;
				synchronized(iterator) {
					if (iterator.hasNext()) {
						e = iterator.next();
					} else {
						return;
					}
				}
				processor.processElement(e);
			}
		}
	}

}