package com.kccoder.fracture.example;

import com.kccoder.fracture.Fracture;
import com.kccoder.fracture.IProcessor;
import com.kccoder.fracture.util.MethodTimer;


public class MandelbrodtExample {
	public static void main(String[] args) throws Exception {
		MethodTimer.compareMethods(MandelbrodtExample.class, "normalWay", "fractureWay");
	}	
	
	public static void normalWay() {
		for(int x = 0; x < 1280; x++) {
			mandelbrot(x);
		}
	}
	
	public static void fractureWay() {
		Fracture.forEach(0, 1280, 1, new IProcessor<Integer>() {
			public void processElement(Integer x) {
				mandelbrot(x);
				
			}			
		});
	}
	
	private static final double factor = 1.5;
	public static void mandelbrot(int sx) {
		for(int sy = 0; sy < 800; sy++) {
			double x0 = (sx-640)/(640/factor);
			double y0 = (sy-400)/(400/factor);
			
			double x = x0;
			double y = y0;
			
			for(int i = 0; i < 1000; i++) {
				if ( (x*x + y*y) >= 4) {
					break;
				}				
				
				double xt = x*x - y*y + x0;
				double yt = 2*x*y + y0;

				x = xt;
				y = yt;	
			}	
		}
	}
}